import test from 'ava';

import { SYNC_STATE, persistentReducer } from '../src';

test('throws if no reducer is provided.', t => {
  t.plan(1);
  t.throws(persistentReducer);
});

test('throws if no reducerName is provided.', t => {
  t.plan(1);
  t.throws(() => persistentReducer(noop => noop));
});

test('returns higher order reducer.', t => {
  t.plan(2);
  const reducer = (state, action) => { };
  const higherOrderReducer = persistentReducer(reducer, 'reducerName');
  t.is(typeof higherOrderReducer, 'function', 'result is function.');
  t.is(higherOrderReducer.length, 2, 'result takes two arguments.');
});

test('higher order reducer replaces state on SYNC_STATE for reducerName.', t => {
  t.plan(3);
  const reducerName = 'reducerName';
  const initialState = { initial: true };
  const replacedState = { initial: false };
  const reducer = (state, action) => {
    t.is(action.type, SYNC_STATE, 'type is SYNC_STATE.');
    t.is(action.payload.name, reducerName, 'name is reducerName.');
    t.is(state, replacedState, 'state replaced.');
  };
  const higherOrderReducer = persistentReducer(reducer, reducerName);
  higherOrderReducer(initialState, { type: SYNC_STATE, payload: { name: reducerName, state: replacedState } });
});

test('higher order reducer maintains state on SYNC_STATE for other reducerName.', t => {
  t.plan(3);
  const reducerName = 'reducerName';
  const otherReducerName = 'otherReducerName';
  const initialState = { initial: true };
  const replacedState = { initial: false };
  const reducer = (state, action) => {
    t.is(action.type, SYNC_STATE, 'type is SYNC_STATE.');
    t.not(action.payload.name, reducerName, 'name is not reducerName.');
    t.is(state, initialState, 'state maintained.');
  };
  const higherOrderReducer = persistentReducer(reducer, reducerName);
  higherOrderReducer(initialState, { type: SYNC_STATE, payload: { name: otherReducerName, state: replacedState } });
});

test('higher order reducer maintains state when type is not SYNC_STATE.', t => {
  t.plan(2);
  const reducerName = 'reducerName';
  const NOT_SYNC_STATE = 'NOT_SYNC_STATE';
  const initialState = { initial: true };
  const reducer = (state, action) => {
    t.not(action.type, SYNC_STATE, 'type is not SYNC_STATE.');
    t.is(state, initialState, 'state maintained.');
  };
  const higherOrderReducer = persistentReducer(reducer, reducerName);
  higherOrderReducer(initialState, { type: NOT_SYNC_STATE });
});

test('allows alternate state comparison method.', t => {
  t.plan(3);
  const reducerName = 'reducerName';
  const initialState = { id: '1', name: 'INITIAL' };
  const replacedState = { id: '1', name: 'REPLACED' };
  const alternateIsEqual = (a, b) => a.id === b.id;
  const reducer = (state, action) => {
    t.is(action.type, SYNC_STATE, 'type is SYNC_STATE.');
    t.is(action.payload.name, reducerName, 'name is reducerName.');
    t.is(state, initialState, 'state was not replaced.');
  };
  const higherOrderReducer = persistentReducer(reducer, reducerName, { stateIsEqual: alternateIsEqual });
  higherOrderReducer(initialState, { type: SYNC_STATE, payload: { name: reducerName, state: replacedState } });
});

test('allows mapping of state.', t => {
  t.plan(7);
  const reducerName = 'reducerName';
  const initialState = { id: '1', name: 'INITIAL' };
  const replacedState = { id: '2', name: 'REPLACED' };
  const mapState = (dbState, storeState) => {
    t.is(replacedState, dbState, 'dbState is passed.');
    t.is(initialState, storeState, 'storeState is passed.');
    return { id: storeState.id, name: replacedState.name };
  };
  const reducer = (state, action) => {
    t.is(action.type, SYNC_STATE, 'type is SYNC_STATE.');
    t.is(action.payload.name, reducerName, 'name is reducerName.');
    t.not(state, initialState, 'state is not storeState.');
    t.not(state, replacedState, 'state is not dbState.');
    t.deepEqual(state, { id: '1', name: 'REPLACED' }, 'state was mapped.');
  };
  const higherOrderReducer = persistentReducer(reducer, reducerName, { mapState });
  higherOrderReducer(initialState, { type: SYNC_STATE, payload: { name: reducerName, state: replacedState } });
});
